@@ -1,5 +1,6 @@
-/* icoutils - A set of programs dealing with MS Windows icons and cursors.
- * Copyright (C) 1998-1999 Oskar Liljeblad
+/* restable.c - Decoding PE and NE resource tables
+ *
+ * Copyright (C) 1998-2005 Oskar Liljeblad
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +18,9 @@
  */
 
 #include <config.h>
+#include "gettext.h"			/* Gnulib */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "common/intparse.h"
 #include "common/memory.h"
 #include "common/error.h"
@@ -33,6 +37,8 @@ static int calc_vma_size (WinLibrary *);
 static void do_resources_recurs (WinLibrary *, WinResource *, WinResource *, WinResource *, WinResource *, char *, char *, char *, DoResourceCallback);
 static char *get_resource_id_quoted (WinResource *);
 static WinResource *find_with_resource_array(WinLibrary *, WinResource *, char *);
+static WinResource *list_resources (WinLibrary *fi, WinResource *res, int *count);
+static bool compare_resource_id (WinResource *wr, char *id);
 
 /* do_resources:
  *   Do something for each resource matching type, name and lang.
@@ -111,10 +117,10 @@ print_resources_callback (WinLibrary *fi, WinResource *wr,
 	if (offset == NULL)
 		return;
 
-	printf("--type=%s --name=%s%s%s [%s%s%soffset=0x%x size=%d]\n",
+	printf(_("--type=%s --name=%s%s%s [%s%s%soffset=0x%x size=%d]\n"),
 	  get_resource_id_quoted(type_wr),
 	  get_resource_id_quoted(name_wr),
-	  (lang_wr->id[0] != '\0' ? " --language=" : ""),
+	  (lang_wr->id[0] != '\0' ? _(" --language=") : ""),
 	  get_resource_id_quoted(lang_wr),
 	  (type != NULL ? "type=" : ""),
 	  (type != NULL ? type : ""),
@@ -135,7 +141,7 @@ get_resource_id_quoted (WinResource *wr)
 	return tmp;
 }
 
-bool
+static bool
 compare_resource_id (WinResource *wr, char *id)
 {
 	if (wr->numeric_id) {
@@ -307,6 +313,10 @@ list_ne_type_resources (WinLibrary *fi, int *count)
 	typeinfo = (Win16NETypeInfo *) fi->first_resource;
 	RETURN_IF_BAD_POINTER(NULL, *typeinfo);
 	for (rescnt = 0 ; typeinfo->type_id != 0 ; rescnt++) {
+		if (((char *) NE_TYPEINFO_NEXT(typeinfo))+sizeof(uint16_t) > fi->memory + fi->total_size) {
+		    warn(_("%s: resource table invalid, ignoring remaining entries"), fi->name);
+		    break;
+		}
 		typeinfo = NE_TYPEINFO_NEXT(typeinfo);
 		RETURN_IF_BAD_POINTER(NULL, *typeinfo);
 	}
@@ -337,7 +347,7 @@ list_ne_type_resources (WinLibrary *fi, int *count)
  *   Return an array of WinResource's in the current
  *   resource level specified by res.
  */
-WinResource *
+static WinResource *
 list_resources (WinLibrary *fi, WinResource *res, int *count)
 {
 	if (res != NULL && !res->is_directory)
@@ -356,9 +366,9 @@ list_resources (WinLibrary *fi, WinResource *res, int *count)
 }
 
 /* read_library:
- *   Read header and get resource directory offset in a Windows library
- *    (AKA module).
  *
+ * Read header and get resource directory offset in a Windows library
+ * (AKA module).
  */
 bool
 read_library (WinLibrary *fi)
@@ -370,26 +380,29 @@ read_library (WinLibrary *fi)
 
 		RETURN_IF_BAD_POINTER(false, mz_header->lfanew);
 		if (mz_header->lfanew < sizeof (DOSImageHeader)) {
-			warn("%s: not a Windows library", fi->name);
+			warn(_("%s: not a PE or NE library"), fi->name);
 			return false;
 		}
+
+		/* falls through */
 	}
 
 	/* check for OS2 (Win16) header signature `NE' */
 	RETURN_IF_BAD_POINTER(false, NE_HEADER(fi->memory)->magic);
 	if (NE_HEADER(fi->memory)->magic == IMAGE_OS2_SIGNATURE) {
 		OS2ImageHeader *header = NE_HEADER(fi->memory);
+		uint16_t *alignshift;
 
 		RETURN_IF_BAD_POINTER(false, header->rsrctab);
 		RETURN_IF_BAD_POINTER(false, header->restab);
 		if (header->rsrctab >= header->restab) {
-			warn("%s: no resource directory found", fi->name);
+			warn(_("%s: no resource directory found"), fi->name);
 			return false;
 		}
 
 		fi->is_PE_binary = false;
-		fi->first_resource = (uint8_t *) NE_HEADER(fi->memory)
-		  + header->rsrctab + sizeof(uint16_t);
+		alignshift = (uint16_t *) ((uint8_t *) NE_HEADER(fi->memory) + header->rsrctab);
+		fi->first_resource = ((uint8_t *) alignshift) + sizeof(uint16_t);
 		RETURN_IF_BAD_POINTER(false, *(Win16NETypeInfo *) fi->first_resource);
 
 		return true;
@@ -438,7 +451,7 @@ read_library (WinLibrary *fi)
 		RETURN_IF_BAD_POINTER(false, pe_header->optional_header.data_directory[IMAGE_DIRECTORY_ENTRY_RESOURCE]);
 		dir = pe_header->optional_header.data_directory + IMAGE_DIRECTORY_ENTRY_RESOURCE;
 		if (dir->size == 0) {
-			warn("%s: file contains no resources", fi->name);
+			warn(_("%s: file contains no resources"), fi->name);
 			return false;
 		}
 
@@ -448,7 +461,7 @@ read_library (WinLibrary *fi)
 	}
 
 	/* other (unknown) header signature was found */
-	warn("%s: not a Windows library", fi->name);
+	warn(_("%s: not a PE or NE library"), fi->name);
 	return false;
 }
 
@@ -462,23 +475,24 @@ calc_vma_size (WinLibrary *fi)
     Win32ImageSectionHeader *seg;
     int c, segcount, size;
 
-	size = 0;
-	RETURN_IF_BAD_POINTER(-1, PE_HEADER(fi->memory)->file_header.number_of_sections);
-	segcount = PE_HEADER(fi->memory)->file_header.number_of_sections;
+    size = 0;
+    RETURN_IF_BAD_POINTER(-1, PE_HEADER(fi->memory)->file_header.number_of_sections);
+    segcount = PE_HEADER(fi->memory)->file_header.number_of_sections;
 
-	/* If there are no segments, just process file like it is.
-	 * This is (probably) not the right thing to do, but problems
-	 * will be delt with later anyway.
-	 */
-	if (segcount == 0)
-		return fi->total_size;
+    /* If there are no segments, just process file like it is.
+     * This is (probably) not the right thing to do, but problems
+     * will be delt with later anyway.
+     */
+    if (segcount == 0)
+    	return fi->total_size;
 
-	seg = PE_SECTIONS(fi->memory);
-	RETURN_IF_BAD_POINTER(-1, *seg);
+    seg = PE_SECTIONS(fi->memory);
+    RETURN_IF_BAD_POINTER(-1, *seg);
+    
     for (c = 0 ; c < segcount ; c++) {
-		RETURN_IF_BAD_POINTER(0, *seg);
+    	RETURN_IF_BAD_POINTER(0, *seg);
 
-        size = MAX(size, seg->virtual_address + seg->size_of_raw_data);
+    	size = MAX(size, seg->virtual_address + seg->size_of_raw_data);
 		/* I have no idea what misc.virtual_size is for... */
         size = MAX(size, seg->virtual_address + seg->misc.virtual_size);
         seg++;
